/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.compose;

import static org.fortiss.af3.platform.utils.PlatformModelElementFactory.createTransmissionConnection;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.platform.model.TransmissionConnection;
import org.fortiss.af3.platform.raspberry.model.CanBus;
import org.fortiss.af3.platform.raspberry.model.CanConnector;
import org.fortiss.af3.platform.raspberry.model.RaspberryPi;
import org.fortiss.tooling.base.model.element.IHierarchicElement;
import org.fortiss.tooling.kernel.extension.IConnectionCompositor;
import org.fortiss.tooling.kernel.extension.data.IConnectionCompositionContext;

/**
 * Connection compositor for {@link RaspberryPi} to {@link CanBus} connections.
 */
public class RaspberryToCanCompositor implements
		IConnectionCompositor<IHierarchicElement, CanConnector, CanConnector> {
	/** {@inheritDoc} */
	@Override
	public boolean canConnect(CanConnector source, CanConnector target, IHierarchicElement parent,
			IConnectionCompositionContext context) {
		if(source.getOwner() instanceof RaspberryPi && target.getOwner() instanceof CanBus) {
			return source.getOutgoing().isEmpty();
		}
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public boolean connect(CanConnector source, CanConnector target, IHierarchicElement parent,
			IConnectionCompositionContext context) {
		createTransmissionConnection(parent, source, target);
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public boolean canReconnect(EObject connection, CanConnector source, CanConnector target,
			IConnectionCompositionContext context) {
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public boolean reconnect(EObject connection, CanConnector source, CanConnector target,
			IConnectionCompositionContext context) {
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public boolean canDisconnect(EObject connection) {
		if(connection instanceof TransmissionConnection) {
			TransmissionConnection c = (TransmissionConnection)connection;
			return c.getSource() instanceof CanConnector && c.getTarget() instanceof CanConnector;
		}
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public boolean disconnect(EObject connection) {
		if(connection instanceof TransmissionConnection) {
			TransmissionConnection tcon = ((TransmissionConnection)connection);
			tcon.setOwner(null);
			tcon.setSource(null);
			tcon.setTarget(null);
			return true;
		}
		return false;
	}
}
